@@ -91,17 +91,21 @@
 // crate-specific exceptions:
 
 mod data;
+mod frame_data;
 mod merge;
 mod profile_view;
 
 pub use data::*;
+pub use frame_data::{FrameData, FrameMeta, UnpackedFrameData};
 pub use merge::*;
 pub use profile_view::{select_slowest, FrameView, GlobalFrameView};
 
+use parking_lot::Mutex;
 use std::collections::BTreeMap;
-use std::sync::atomic::{AtomicBool, Ordering};
-use std::sync::Arc;
-use std::sync::Mutex;
+use std::sync::{
+    atomic::{AtomicBool, Ordering},
+    Arc,
+};
 
 static MACROS_ON: AtomicBool = AtomicBool::new(false);
 
@@ -321,156 +325,6 @@ pub struct StreamInfoRef<'a> {
 
 // ----------------------------------------------------------------------------
 
-/// One frame worth of profile data, collected from many sources.
-#[derive(Clone)]
-#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
-pub struct FrameData {
-    pub frame_index: FrameIndex,
-    pub thread_streams: BTreeMap<ThreadInfo, Arc<StreamInfo>>,
-    pub range_ns: (NanoSecond, NanoSecond),
-    /// The uncompressed size
-    pub num_bytes: usize,
-    pub num_scopes: usize,
-
-    /// Size of the compressed data (in bytes), if this was decompressed.
-    #[cfg_attr(feature = "serde", serde(default, skip))]
-    pub compressed_size: Option<usize>,
-}
-
-impl FrameData {
-    pub fn new(
-        frame_index: FrameIndex,
-        thread_streams: BTreeMap<ThreadInfo, StreamInfo>,
-    ) -> Result<Self> {
-        let thread_streams: BTreeMap<_, _> = thread_streams
-            .into_iter()
-            .map(|(info, stream_info)| (info, Arc::new(stream_info)))
-            .collect();
-
-        let mut num_bytes = 0;
-        let mut num_scopes = 0;
-
-        let mut min_ns = NanoSecond::MAX;
-        let mut max_ns = NanoSecond::MIN;
-        for stream_info in thread_streams.values() {
-            num_bytes += stream_info.stream.len();
-            num_scopes += stream_info.num_scopes;
-            min_ns = min_ns.min(stream_info.range_ns.0);
-            max_ns = max_ns.max(stream_info.range_ns.1);
-        }
-
-        if min_ns <= max_ns {
-            Ok(Self {
-                frame_index,
-                thread_streams,
-                range_ns: (min_ns, max_ns),
-                num_bytes,
-                num_scopes,
-                compressed_size: None,
-            })
-        } else {
-            Err(Error::Empty)
-        }
-    }
-
-    pub fn duration_ns(&self) -> NanoSecond {
-        let (min, max) = self.range_ns;
-        max - min
-    }
-
-    /// Writes one Frame into a stream, prefixed by it's length (u32 le).
-    #[cfg(feature = "serialization")]
-    pub fn write_into(&self, write: &mut impl std::io::Write) -> anyhow::Result<()> {
-        use bincode::Options as _;
-        let serialized = bincode::options().serialize(self)?;
-
-        if false {
-            let compressed = lz4_flex::compress_prepend_size(&serialized);
-            write.write_all(b"PFD0")?;
-            write.write_all(&(compressed.len() as u32).to_le_bytes())?;
-            write.write_all(&compressed)?;
-        } else {
-            // zstd cuts sizes in half compared to lz4_flex
-            let level = 3;
-            let compressed = zstd::encode_all(std::io::Cursor::new(&serialized), level)?;
-            write.write_all(b"PFD1")?;
-            write.write_all(&(compressed.len() as u32).to_le_bytes())?;
-            write.write_all(&compressed)?;
-        }
-        Ok(())
-    }
-
-    /// Read the next [`FrameData`] from a stream.
-    ///
-    /// `None` is returned if the end of the stream is reached (EOF),
-    /// or an end-of-stream sentinel of 0u32 is read.
-    #[cfg(feature = "serialization")]
-    pub fn read_next(read: &mut impl std::io::Read) -> anyhow::Result<Option<Self>> {
-        use anyhow::Context as _;
-
-        let mut header = [0_u8; 4];
-        if let Err(err) = read.read_exact(&mut header) {
-            if err.kind() == std::io::ErrorKind::UnexpectedEof {
-                return Ok(None);
-            } else {
-                return Err(err.into());
-            }
-        }
-
-        if header == [0_u8; 4] {
-            Ok(None) // end-of-stream sentinel.
-        } else if header.starts_with(b"PFD") {
-            if &header == b"PFD0" {
-                let mut compressed_length = [0_u8; 4];
-                read.read_exact(&mut compressed_length)?;
-                let compressed_length = u32::from_le_bytes(compressed_length) as usize;
-                let mut compressed = vec![0_u8; compressed_length];
-                read.read_exact(&mut compressed)?;
-
-                let serialized =
-                    lz4_flex::decompress_size_prepended(&compressed).context("lz4 decompress")?;
-
-                use bincode::Options as _;
-                let mut frame: Self = bincode::options()
-                    .deserialize(&serialized)
-                    .context("bincode deserialize")?;
-                frame.compressed_size = Some(compressed_length);
-                Ok(Some(frame))
-            } else if &header == b"PFD1" {
-                let mut compressed_length = [0_u8; 4];
-                read.read_exact(&mut compressed_length)?;
-                let compressed_length = u32::from_le_bytes(compressed_length) as usize;
-                let mut compressed = vec![0_u8; compressed_length];
-                read.read_exact(&mut compressed)?;
-
-                let serialized = zstd::decode_all(&compressed[..]).context("zstd decompress")?;
-
-                use bincode::Options as _;
-                let mut frame: Self = bincode::options()
-                    .deserialize(&serialized)
-                    .context("bincode deserialize")?;
-                frame.compressed_size = Some(compressed_length);
-                Ok(Some(frame))
-            } else {
-                anyhow::bail!("Failed to decode: this data is newer than this reader. Please update your puffin version!");
-            }
-        } else {
-            // Old packet without magic header
-            let mut bytes = vec![0_u8; u32::from_le_bytes(header) as usize];
-            read.read_exact(&mut bytes)?;
-
-            use bincode::Options as _;
-            Ok(Some(
-                bincode::options()
-                    .deserialize(&bytes)
-                    .context("bincode deserialize")?,
-            ))
-        }
-    }
-}
-
-// ----------------------------------------------------------------------------
-
 type NsSource = fn() -> NanoSecond;
 type ThreadReporter = fn(ThreadInfo, &StreamInfoRef<'_>);
 
@@ -598,10 +452,10 @@ impl Default for GlobalProfiler {
 
 impl GlobalProfiler {
     /// Access to the global profiler singleton.
-    pub fn lock() -> std::sync::MutexGuard<'static, Self> {
+    pub fn lock() -> parking_lot::MutexGuard<'static, Self> {
         use once_cell::sync::Lazy;
         static GLOBAL_PROFILER: Lazy<Mutex<GlobalProfiler>> = Lazy::new(Default::default);
-        GLOBAL_PROFILER.lock().unwrap() // panic on mutex poisoning
+        GLOBAL_PROFILER.lock()
     }
 
     /// You need to call this once at the start of every frame.
